@@ -5,9 +5,10 @@ import networkx as nx
 from typing import List, Dict
 
 import ascend.protos.api.api_pb2 as api_pb2
+import ascend.protos.io.io_pb2 as io_pb2
 from ascend.sdk.client import Client
-from ascend.sdk.definitions import (Component, Dataflow, ReadConnector, Transform, WriteConnector, ComponentGroup, ComponentIdToUuidMap, ComponentUuidType,
-                                    DataFeed, DataFeedConnector, DataService)
+from ascend.sdk.definitions import (Component, Connection, Credential, Dataflow, ReadConnector, Transform, WriteConnector, ComponentGroup, ComponentIdToUuidMap,
+                                    ComponentUuidType, DataFeed, DataFeedConnector, DataService)
 from openapi_client.exceptions import ApiException
 
 
@@ -22,7 +23,7 @@ class DataServiceApplier:
     """Creates a new DataServiceApplier."""
     self.client = client
 
-  def apply(self, data_service: DataService, delete=True, dry_run=False):
+  def apply(self, data_service: DataService, delete=True, dry_run=False, after_credentials=None, after_connections=None):
     """
     Create or update the specified DataService.
 
@@ -52,23 +53,120 @@ class DataServiceApplier:
         glog.info(f"Create DataService: {data_service.id}")
         self.client.create_data_service(data_service.to_proto())
 
+    for i, cred in enumerate(data_service.credentials):
+      data_service.credentials[i] = CredentialApplier(self.client).apply(data_service.id, cred, dry_run)
+    if after_credentials:
+      after_credentials()
+
+    for i, conn in enumerate(data_service.connections):
+      data_service.connections[i] = ConnectionApplier(self.client).apply(data_service.id, conn, dry_run)
+    if after_connections:
+      after_connections()
+
     for df in data_service.dataflows:
       DataflowApplier(self.client).apply(data_service.id, df, delete, dry_run)
 
     try:
       if delete:
-        expected = [dataflow.id for dataflow in data_service.dataflows]
-        # TODO - does not yet account for dependencies between dataflows
-        for df in self.client.list_dataflows(data_service.id).data:
-          if df.id not in expected:
-            glog.info(f"Delete Dataflow: (ds={data_service.id} df={df.id})")
-            if not dry_run:
-              self.client.delete_dataflow(data_service.id, df.id)
+        self._sweep(data_service, dry_run)
     except ApiException as e:
       # tolerate 404s during dry runs, since they're likely to happen
       if e.status != 404 or not dry_run:
         raise e
 
+  def _sweep(self, data_service: DataService, dry_run=False):
+    """ Delete any dataflows, credential, or connections that are not part of 'data_service'
+    """
+    expected = [dataflow.id for dataflow in data_service.dataflows]
+    # TODO - does not yet account for dependencies between dataflows
+    for df in self.client.list_dataflows(data_service.id).data:
+      if df.id not in expected:
+        glog.info(f"Delete Dataflow: (ds={data_service.id} df={df.id})")
+        if not dry_run:
+          self.client.delete_dataflow(data_service.id, df.id)
+
+    # TODO - decide on what uniquely defines connections
+    expected = [connection.name for connection in data_service.connections]
+    for connection in self.client.list_connections(data_service.id).data:
+      if connection.name not in expected:
+        glog.info(f"Delete Connection: (ds={data_service.id} id={connection.id.value} {connection.name})")
+        if not dry_run:
+          self.client.delete_connection(data_service.id, connection.id.value)
+
+    expected = [credential.name for credential in data_service.credentials]
+    for credential in self.client.list_data_service_credentials(data_service.id).data:
+      if credential.name not in expected:
+        glog.info(f"Delete Credential: (ds={data_service.id} credential_id={credential.credential_id} {credential.name})")
+        if not dry_run:
+          self.client.delete_data_service_credentials(data_service.id, credential.credential_id)
+
+
+class CredentialApplier:
+  def __init__(self, client: Client):
+    self.client = client
+
+  def apply(self, data_service_id, credential: Credential, dry_run=False):
+    glog.info(f"Apply Credential: (ds={data_service_id} credential_id={credential.id} name={credential.name})")
+    existing = None
+
+    # There is no "get credential" call, so this is a bit ineficient
+    for c in self.client.list_data_service_credentials(data_service_id).data:
+      if credential.id == c.credential_id or credential.name == c.name:
+        existing = c
+        break
+
+    credential_p = credential.to_proto()
+    if existing and existing.credential_id != "":
+      credential.id = credential.credential.id.value = existing.credential_id
+      glog.info(f"Update Credential: (ds={data_service_id} credential_id={credential.id} name={credential.name})")
+      if not dry_run:
+        existing = self.client.update_data_service_credentials(data_service_id, credential.id, credential_p).data
+    else:
+      credential.id = credential.credential.id.value = ""
+      glog.info(f"Create Credential: (ds={data_service_id} credential_id={credential.id} name={credential.name})")
+      if not dry_run:
+        existing = self.client.create_data_service_credentials(data_service_id, credential_p).data
+
+    if existing:
+      # Special copy here to preserve cred values in the object we return
+      credential_p.credential_id = existing.credential_id
+      credential_p.credential.id.CopyFrom(existing.credential.id)
+    return Credential.from_proto(credential_p)
+
+
+class ConnectionApplier:
+  def __init__(self, client: Client):
+    self.client = client
+
+  def apply(self, data_service_id, connection: Connection, dry_run=False):
+    glog.info(f"Apply Connection: (ds={data_service_id} id={connection.id} name={connection.name})")
+    existing = None
+
+    # we have no unique ID, so just go by name for now... additionally, updating fails,
+    # so if we find something with the same name, just return that
+    for c in self.client.list_connections(data_service_id).data:
+      if connection.id == c.id or connection.name == c.name:
+        existing = c
+        break
+
+    if existing and existing.id.value != "":
+      connection.id = existing.id.value
+      glog.info(f"Update Connection: (ds={data_service_id} id={connection.id} name={connection.name})")
+      if not dry_run:
+        res = self.client.update_connection(data_service_id, connection.id, connection.to_proto())
+        # glog.info(res)
+        # short term hack as we aren't getting full results from the API on creation (See FP2021-49)
+        existing = self.client.get_connection(data_service_id, res.data.value).data
+    else:
+      connection.id = ""
+      glog.info(f"Create Connection: (ds={data_service_id} id={connection.id} name={connection.name})")
+      if not dry_run:
+        existing = self.client.create_connection(data_service_id, connection.to_proto()).data
+
+    if existing:
+      return Connection.from_proto(existing)
+    return connection
+
 
 class DataflowApplier:
   """DataflowApplier is a utility class that accepts a Dataflow definition and 'applies'
@@ -267,7 +365,7 @@ class DataFeedApplier:
 
     ds_uuid_to_id = {ds.uuid: ds.id for ds in self.client.list_data_services().data}
     roles = self.client.list_data_service_roles().data
-    ds_to_role_map = {ds_uuid_to_id[role.org_id]: role.uuid for role in roles if role.id == 'Everyone'}
+    ds_to_role_map = {ds_uuid_to_id[role.org_id]: role.uuid for role in roles if role.id == 'Everyone' and role.org_id in ds_uuid_to_id}
 
     if exists:
       glog.info(f"Update DataFeed: (ds={data_service_id} df={dataflow_id} {data_feed.id})")
