@@ -10,8 +10,8 @@ from typing import Dict
 
 import ascend.protos.api.api_pb2 as api_pb2
 from ascend.sdk.client import Client
-from ascend.sdk.definitions import (Dataflow, ReadConnector, Transform, WriteConnector, ComponentGroup, ComponentUuidToIdMap, DataFeed, DataFeedConnector,
-                                    DataService)
+from ascend.sdk.definitions import (Credential, Connection, Dataflow, ReadConnector, Transform, WriteConnector, ComponentGroup, ComponentUuidToIdMap, DataFeed,
+                                    DataFeedConnector, DataService)
 
 
 def data_service_from_proto(client: Client, proto: api_pb2.DataService) -> 'DataService':
@@ -26,12 +26,14 @@ def data_service_from_proto(client: Client, proto: api_pb2.DataService) -> 'Data
   - proto: DataService protobuf object
   Returns: a DataService definition
   """
-  dataflow_protos = client.list_dataflows(proto.id).data
   return DataService(
       id=proto.id,
       name=proto.name,
       description=proto.description,
-      dataflows=[dataflow_from_proto(client, proto.id, dataflow_proto) for dataflow_proto in dataflow_protos],
+      credentials=sorted([Credential.from_proto(credential_proto) for credential_proto in client.list_data_service_credentials(proto.id).data],
+                         key=lambda c: c.name),
+      connections=sorted([Connection.from_proto(connection_proto) for connection_proto in client.list_connections(proto.id).data], key=lambda c: c.name),
+      dataflows=sorted([dataflow_from_proto(client, proto.id, dataflow_proto) for dataflow_proto in client.list_dataflows(proto.id).data], key=lambda c: c.id),
   )
 
 
