@@ -40,10 +40,11 @@ class InlineCode:
     self.base64_encoded = base64_encoded
 
   def loader(self) -> str:
+    rel_path_components = ", ".join(map(lambda x: f'"{x}"', pathlib.Path(self._rel_path).parts))
     if self.base64_encoded:
-      return f'''base64.b64encode(pathlib.Path(os.path.join(os.path.dirname(os.path.realpath(__file__)), "{self._rel_path}")).read_bytes()).decode()'''
+      return f'''base64.b64encode(pathlib.Path(os.path.join(os.path.dirname(os.path.realpath(__file__)), {rel_path_components})).read_bytes()).decode()'''
     else:
-      return f'''pathlib.Path(os.path.join(os.path.dirname(os.path.realpath(__file__)), "{self._rel_path}")).read_text()'''
+      return f'''pathlib.Path(os.path.join(os.path.dirname(os.path.realpath(__file__)), {rel_path_components})).read_text()'''
 
 
 def download_dataflow(client: Client, data_service_id: str, dataflow_id: str, resource_base_path: str = "."):
@@ -101,7 +102,10 @@ def download_data_service(client: Client, data_service_id: str, resource_base_pa
 
   ds_proto = client.get_data_service(data_service_id=data_service_id).data
   data_service = data_service_from_proto(client, ds_proto)
+  write_data_service(client.hostname, data_service, resource_base_path)
 
+
+def write_data_service(hostname: str, data_service: DataService, resource_base_path: str = "."):
   data_service_path = os.path.join(resource_base_path, f"{data_service.id}.py")
   dataflow_resource_paths = [os.path.join(resource_base_path, f"{dataflow.id}") for dataflow in data_service.dataflows]
   glog.info(f"Writing data service definition to ({data_service_path})" f" and dataflow resource files to ({','.join(dataflow_resource_paths)})")
@@ -109,7 +113,7 @@ def download_data_service(client: Client, data_service_id: str, resource_base_pa
   for path in dataflow_resource_paths:
     pathlib.Path(path).mkdir(parents=True, exist_ok=True)
 
-  rendered_data_service, inline_code_list = render_data_service(data_service, client.hostname, resource_base_path)
+  rendered_data_service, inline_code_list = render_data_service(data_service, hostname, resource_base_path)
   for inline_code in inline_code_list:
     with open(inline_code.resource_path, "w") as f:
       f.write(inline_code.code)
@@ -185,7 +189,9 @@ def render_data_service(data_service: DataService, hostname: str, resource_path:
   return rendered, inline_code_list
 
 
-_proto_mods = ["ascend", "component", "content_encoding", "core", "expression", "format", "function", "io", "operator", "pattern", "schema", "text"]
+_proto_mods = [
+    "ascend", "component", "connection", "content_encoding", "core", "expression", "format", "function", "io", "operator", "pattern", "schema", "text"
+]
 
 _gmod_classes = [("google.protobuf.wrappers_pb2", "DoubleValue"), ("google.protobuf.wrappers_pb2", "BoolValue"), ("google.protobuf.wrappers_pb2", "Int64Value"),
                  ("google.protobuf.wrappers_pb2", "UInt64Value"), ("google.protobuf.wrappers_pb2", "Int32Value"),
@@ -196,7 +202,9 @@ _gmod_classes = [("google.protobuf.wrappers_pb2", "DoubleValue"), ("google.proto
 def _classname_map() -> dict:
   classname_map: dict = {}
 
-  for defclass in ['DataService', 'Dataflow', 'ReadConnector', 'WriteConnector', 'Transform', 'ComponentGroup', 'DataFeed', 'DataFeedConnector']:
+  for defclass in [
+      'DataService', 'Credential', 'Connection', 'Dataflow', 'ReadConnector', 'WriteConnector', 'Transform', 'ComponentGroup', 'DataFeed', 'DataFeedConnector'
+  ]:
     classname_map[defclass] = f"definitions.{defclass}"
 
   for _, cls in _gmod_classes:
@@ -293,7 +301,7 @@ def _render_value(val, attribute_path: Tuple[str, ...], indent=0, classname_map:
   elif isinstance(val, Definition):
     return _render_definition(val, attribute_path, indent, classname_map, attribute_overrides)
   elif isinstance(val, str):
-    return f"'''{val}'''"
+    return f"r'''{val}'''"
   else:
     return val
 
@@ -339,7 +347,7 @@ def _render_message(message: ProtoMessage, attribute_path: Tuple[str, ...], inde
     elif field_descriptor.type == descriptor.FieldDescriptor.TYPE_MESSAGE:
       return _render_message(val, attribute_path, indent, classname_map, attribute_overrides)
     elif field_descriptor.type == descriptor.FieldDescriptor.TYPE_STRING:
-      return f"'''{val}'''"
+      return f"r'''{val}'''"
     else:
       return val
 
@@ -371,6 +379,8 @@ def _spaces_for(indent):
 
 
 def _components_ordered_by_dependency(components: List[Component]) -> List[Component]:
+  # return sorted(components, key=lambda c: c.name)
+
   ordered: List[Component] = []
 
   id_to_component = _component_id_map_from_list(components)
